// sprite.cpp
//
// -- This class encapsulates sprites.
// -- What I want to do is remove from the game engine any sprite movement specifics.
// -- The sprite is an object, and should know what it wants to do, and how to
// -- do it.  All the engine should do is pass information to the sprite about
// -- the environment.  The sprite should give information back to the engine
// -- about itself. Derive your own sprite objects and add the functionality
// -- you need
// --

#include <owl\module.h>
#include <owl\point.h>
#include <owl\gdiobjec.h>
#include <owl\window.h>
#include <classlib\arrays.h>
#pragma hdrstop
#include "wingdc.h"
#include "buffer.h"
#include "sprite.h"

// Default constructor.  This is needed for TArray template
Sprite::Sprite(){
	loc = TRect(0,0,0,0);
	size = TSize(0,0);
	oldLoc = TRect(0,0,0,0);
	vx = 0;
	vy = 0;
	numFrames = 0;
	currFrame = 0;
	transColor = 0;
	state = 0;
	animClock = 0;
	animSpeed = 0;
	motionClock = 0;
	motionSpeed = 0;
	memset (cells, 0, sizeof cells);
	isDirty = false;
}

Sprite::Sprite(TRect cLoc,
				BYTE huge *bits,
				int cVx,
				int cVy,
				int cNumFrames,
				int cCurrFrame,
				int cTransColor,
				bool cIsDirty,
				int cState ,
				int cAnimClock,
				int cAnimSpeed,
				int cMotionClock,
				int cMotionSpeed){

	loc = cLoc;
	size.cx = loc.Width();
	size.cy = loc.Height();
	oldLoc = loc;
	isDirty = cIsDirty;
	vx = cVx;
	vy = cVy;
	numFrames = cNumFrames;
	currFrame = cCurrFrame;
	transColor = cTransColor;
	state = cState;
	animClock = cAnimClock;
	animSpeed = cAnimSpeed;
	motionClock = cMotionClock;
	motionSpeed = cMotionSpeed;

	// Build the array of pointer to the animation cells.
	// You should build your cells one stacked on top of each other
	// with a single line of pixels seperating them.  The first frame
	// of the animation should be on the bottom, with subsequent ones stacked
	// on it. ALL CELLS MUST BE THE SAME HEIGHT AND THE SAME WIDTH.  For example
	// you might have 5 cells, each being 10 x 16.  You Dib would then be
	// 10 x 84.  Height of 84 because (5 cells * 16) + (4 seperation lines).


	memset (cells, 0, sizeof cells);

	if (numFrames > MAX_ANIM_CELLS)
		 numFrames = MAX_ANIM_CELLS;

	int x;
	for (x = 0; x < numFrames; x++){
		 cells[x] = bits;
		 bits += (size.cx * (size.cy + 1)); // add 1 for the seperator line
	}
}

bool Sprite::Draw(TBuffer &buffer){

	return _ASM_TransCopyDIBBits(loc.left,loc.top,
									size.cx,size.cy,
									cells[currFrame],
									buffer.nWidth,buffer.nHeight,
									(BYTE huge *)buffer.pOffScreenBits,
									buffer.lBMPOrientation,transColor);
}

