// sprite.h
//
// -- Offers very basic sprite functionality.

#if !defined(__SPRITE_XX_H)
#define __SPRITE_XX_H

#include <owl\module.h>
#include <owl\point.h>
#include <owl\gdiobjec.h>
#include <owl\window.h>
#include <classlib\arrays.h>

#pragma hdrstop
#include "wingdc.h"
#include "buffer.h"

#define MAX_ANIM_CELLS (15)

extern "C" BOOL far _ASM_TransCopyDIBBits(long lDrawX, long lDrawY,
					long lSrcWidth, long lSrcHeight, BYTE huge *hpSrcBits,
					long lDstWidth, long lDstHeight, BYTE huge *hpDstBits,
								  long lOrientation, BYTE bTransparentColor);

// Draw your sprite cells stacked one on top of the other, with a single row of
// pixels between each one.  Call the constructor with the Dibs source bits address.
// and the number of cells in the Dib.  Assumes Dibs are bottom to top format. The
// The loc rectangle should hold the rect where the sprite should initially appear.
// The constructor uses the loc.Height() and loc.Width() to build the array of
// pointers to the cells.  For Current frame, send as with arrays: 0 = first cell,
// 1 = second cell, etc, etc.

class Sprite {

  public:

	  // Need default constructor in order to use TArray
	  Sprite();

	  Sprite(TRect cLoc,
				BYTE huge *bits,
				int cVx,
				int cVy,
				int cNumFrames,
				int cCurrframe,
				int cTransColor,
				bool cIsDirty = true,
				int cState = 0,
				int cAnimClock = 0,
				int cAnimSpeed = 0,
				int cMotionClock = 0,
				int cMotionSpeed = 0);


	  TRect loc; // keep location in a rect to help in finding unions
					 // and intersections
	  TSize size; // get this from the loc rect.  Size is constant.
	  TRect oldLoc; // Use this to ??
	  int vx;  // x velocity
	  int vy;  // y velocity
	  bool isDirty; // Does the sprite need to be redrawn?

	  // Use these to trigger cell changes
	  int animClock;
	  int animSpeed;

	  // use these to scale speed
	  int motionClock;
	  int motionSpeed;

	  // Animation cell stuff
	  int currFrame; // current animation frame;
	  int numFrames; // number of animation frames. Cant exceed MAX_ANIM_CELLS
	  BYTE huge *cells[MAX_ANIM_CELLS]; // Pointers to the Dibs bits

	  int state; // Sprites state.  Alive, dead, hidden ... ? ?

	  int transColor;      // Contains the index of the palette entry you want
								  // to be the t4ransparent color.

	  // this is needed for the TArry template
		int operator ==(const Sprite & sprite) const{
			return &sprite == this;
		}

		inline bool Draw(TBuffer &);  // copy the sprite to the WinG buffer

};


#endif // #if !defined(__SPRITE_XX_H)

