//--
//--	spritwin.cpp
//--
//--	SpriteWindow Class Module
//--  This is where your action is.  It inherits Frame window functionality
//    and a WinG buffer object from TBitmapAnimatorWindow.  It uses the Sprite class
//    to build a TArray of sprites, then move them around on the screen.
//
//    All I am using is one DIB that contains 6 animation cells.  I create a
//    container of sprites, assigning a random cell to each sprite.  I also
//    assign a random velocity to each.  Then I just move them around
//--

#include <stdlib.h>
#include <time.h>
#include <owl\window.h>
#include <owl\framewin.h>
#include <owl\module.h>
#include <owl\point.h>
#include <owl\dc.h>
#include <owl\color.h>
#include <owl\gdiobjec.h>
#include <stdlib.h>
#include <time.h>
#pragma hdrstop
#include "wingdc.h"
#include "animwind.h"
#include "spritwin.h"
#include "sprite.h"
#include "engine.rh"
#include "graph1.h"

DEFINE_RESPONSE_TABLE1(SpriteWindow, TBitmapAnimatorWindow)
	EV_WM_LBUTTONDOWN,
	EV_WM_RBUTTONDOWN,
	EV_WM_KILLFOCUS,
END_RESPONSE_TABLE;


//----------------------------
//  SpriteWindow::SpriteWindow()
//----------------------------
SpriteWindow::SpriteWindow(TWindow *pParWin,
								  const char far *pszTitle) :
					 TBitmapAnimatorWindow(pParWin,pszTitle)
{
	pBkgndDC = 0;  // Allocated in SetupWindow()
	pBkgndBitmap = 0;  // Allocated in SetupWindow()

	backgroundROP = BLACKNESS; // Overide base class
	spriteDibs[0] = new TDib(GetApplication()->GetInstance(), IDB_DOTS);
	numbDibs = 1;
	cont = false;

}


//-----------------------------
//  SpriteWindow::~SpriteWindow()
//-----------------------------
SpriteWindow::~SpriteWindow()
{


	delete pBkgndDC;
	delete pBkgndBitmap;

	for (int x = 0; x < numbDibs; x++)
		delete spriteDibs[x];

	delete sprites;
	delete dirtyRects;

}

void SpriteWindow::EvKillFocus(HWND hwnd){

	 TWindow::EvKillFocus(hwnd);
}


//-----------------------------
//  SpriteWindow::SetupWindow()
//-----------------------------
void SpriteWindow::SetupWindow()
{
	TBitmapAnimatorWindow::SetupWindow();

	// Load background DIB as a bitmap and select it into a memory DC for
	//   fast blts to the offscreen buffer (WinGDC)
	if (buffer->pBufferDC)
	{
		pBkgndBitmap = new TBitmap(*(buffer->pBufferDC),
											TDib(GetApplication()->GetInstance(),IDB_BACK));
		pBkgndDC = new TMemoryDC(*(buffer->pBufferDC));
		pBkgndDC->SelectObject(*pPalette);//??? Why is this needed.  I commented it out, and it still works
		pBkgndDC->SelectObject(*pBkgndBitmap);
		BkgndRect.SetNull();
	}

		// Center the background bitmap
	BkgndRect.left = (clientRect.Width() - pBkgndBitmap->Width()) / 2;
	BkgndRect.top = (clientRect.Height() - pBkgndBitmap->Height()) / 2;
	BkgndRect.right = BkgndRect.left + pBkgndBitmap->Width();
	BkgndRect.bottom = BkgndRect.top + pBkgndBitmap->Height();
	DrawBackground(clientRect); // initialize the buffer

	randomize();
	sprites = new Sprites(400,0,50);
	BuildSpriteList();
	dirtyRects = new RectList(10,0,10);


}


void SpriteWindow::BuildSpriteList(){

	sprites->Flush();
	numbSprites = 250;
	int x,y,c;

	for (c = 0; c < numbSprites; c++){
	  x = random(clientRect.Width());
	  y = random(clientRect.Height());

	  //  Create the sprites, assigning each one a random frame of the DIB.
	  //  Notice that the sprite loc.right and loc.bottom are calculated by
	  //  adding the height and width of the cell (both are 16 in this case).

	  Sprite sp(TRect(x,y,x + 16, y + 16),(BYTE huge *)spriteDibs[0]->GetBits(),
					  // assign a random x velocity from -3 to 3
					  (random (2) + 1) * (random(2) ? -1 : 1),
					  // assign a random y velocity from -3 to 3
					  (random (2) + 1) * (random(2) ? -1 : 1),
					  6, // Number of frames in bitmap
					  random(6),// assign a random frame
					  237); // Tansparent color index

	  sprites->Add(sp);
	}

}

bool SpriteWindow::CanClose(){

	 Melt(*pClientDC, clientRect, TColor::LtGray,0);
	 return true;
}

//-----------------------------
//  SpriteWindow::InitPalette()
//-----------------------------
void SpriteWindow::CleanupWindow(){

	 cont = false;
	 TBitmapAnimatorWindow::CleanupWindow();
}

void SpriteWindow::InitPalette()
{
	// Allocate and init a temporary PALETTEENTRY array
	uint nNumEntries = pPalette->GetNumEntries();
	PALETTEENTRY *pPalEntries = new PALETTEENTRY[nNumEntries];

	// fill the array with the palette entries
	pPalette->GetPaletteEntries(0,nNumEntries,pPalEntries);

	// copy palette entries from pSpriteDIBs color table
	TRgbQuad *pDIBColors = spriteDibs[0]->GetColors();

	for (uint nCounter = 10; nCounter < 246; ++nCounter)
	{
		pPalEntries[nCounter].peRed = pDIBColors[nCounter].rgbRed;
		pPalEntries[nCounter].peGreen = pDIBColors[nCounter].rgbGreen;
		pPalEntries[nCounter].peBlue = pDIBColors[nCounter].rgbBlue;
	}

	// place the entries back in the palette
	pPalette->SetPaletteEntries(0,nNumEntries,pPalEntries);

	// Free the temporary PALETTEENTRY array
	delete pPalEntries;
}


//------------------------------------
//  SpriteWindow::DrawBackground()
//------------------------------------
void SpriteWindow::DrawBackground(TRect &UpdateRect)
{
	// Clear the updateRect portion of the background to the ROP color
	buffer->pBufferDC->PatBlt(UpdateRect.left,UpdateRect.top,
							UpdateRect.Width(),UpdateRect.Height(),backgroundROP);

	// Copy the intersection of the background bitmap and the updateRect
	TRect BltRect = UpdateRect & BkgndRect;// Get their intersection
	if (!BltRect.IsEmpty()) // if they intersect, paint the intersection
		buffer->pBufferDC->BitBlt(BltRect,*pBkgndDC,
										  TPoint(BltRect.left - BkgndRect.left,
										  BltRect.top - BkgndRect.top),
										  SRCCOPY);

}

void SpriteWindow::AnimLoop(){

// I am using small sprites and they are all moving, so my rendering is
// simple.  If I was using large sprites, or just a few small ones, I may
// want to change the logic to include dirty rectangle processing.

	 while(cont){ // loop is broken when right button is clicked, or
					  // window is closed
		  DrawBackground(clientRect);
		  TransformSprites(); // erase old sprites, and move them
		  UpdateSprites(); //draw each sprite to the buffer
		  UpdateScreen(clientRect); // just Blit the whole buffer to the screen.  If I was only
							 // using a few sprites, it would be faster to track
							 // dirty rectangles.

		  GetApplication()->PumpWaitingMessages();
  }
}


void SpriteWindow::TransformSprites(){

	 int x;
	 for (x = 0; x < numbSprites; x++){
		 Sprite *sp;
		 sp = &((*sprites)[x]);


		 // change sprites velocities and animation here.
		 // I have constant velocitys and no animation for the sprites,
		 sp->loc.left += sp->vx;
		 sp->loc.top += sp->vy;

		 // Wrap the sprite around the screen
		 if (sp->loc.left < 0 )
			  sp->loc.left = clientRect.right - sp->size.cx + 1;
		 else
			  if (sp->loc.left > clientRect.right)
				  sp->loc.left = 0;
		 if (sp->loc.top < 0 )
			  sp->loc.top = clientRect.bottom - sp->size.cy + 1;
		 else
			  if (sp->loc.top > clientRect.bottom)
				  sp->loc.top = 0;

		 sp->loc.bottom = sp->loc.top + sp->size.cy;
		 sp->loc.right = sp->loc.left + sp->size.cx;
	 }

}

void SpriteWindow::UpdateSprites(){

	 // Draw each sprite into the buffer

	 int x;
	 for (x = 0; x < numbSprites; x++)
		 (*sprites)[x].Draw(*buffer);

}

void SpriteWindow::EvRButtonDown(uint,TPoint &){

	cont = false;

}

void SpriteWindow::EvLButtonDown(uint,TPoint &)
{

	 if (!cont){ // If no animation in progress
		 cont =  true;  // start it up
		 AnimLoop();
	 }

}




