//--
//--	wingdc.cpp
//--
//--	TWinGDC Class Module
//--


#include <owl\owlpch.h>
#include <owl\module.h>
#include <owl\dc.h>
#pragma hdrstop
#include "wingdc.h"


TModule *TWinGDC::pWinGModule = 0;  // Pointer to the WinG DLL
int TWinGDC::nWinGModuleRefCount = 0; // # of TWinGDC objects in existence


//----------------------
//  TWinGDC::TWinGDC()
//----------------------
TWinGDC::TWinGDC() : TDC()
{
	if (!nWinGModuleRefCount)
	{
		// Load/Register WING.DLL Microsoft WinG Graphics Library Module
		try
		{
			#if !defined(__FLAT__)
			pWinGModule = new TModule("WING.DLL", TRUE);
			#else
			pWinGModule = new TModule("WING32.DLL", TRUE);
			#endif

			(FARPROC)__WinGCreateDC =
						pWinGModule->GetProcAddress("WinGCreateDC");
			(FARPROC)__WinGRecommendDIBFormat =
						pWinGModule->GetProcAddress("WinGRecommendDIBFormat");
			(FARPROC)__WinGCreateBitmap =
						pWinGModule->GetProcAddress("WinGCreateBitmap");
			(FARPROC)__WinGGetDIBPointer =
						pWinGModule->GetProcAddress("WinGGetDIBPointer");
			(FARPROC)__WinGGetDIBColorTable =
						pWinGModule->GetProcAddress("WinGGetDIBColorTable");
			(FARPROC)__WinGSetDIBColorTable =
						pWinGModule->GetProcAddress("WinGSetDIBColorTable");
			(FARPROC)__WinGCreateHalftonePalette =
						pWinGModule->GetProcAddress("WinGCreateHalftonePalette");
			(FARPROC)__WinGCreateHalftoneBrush =
						pWinGModule->GetProcAddress("WinGCreateHalftoneBrush");
			(FARPROC)__WinGBitBlt =
						pWinGModule->GetProcAddress("WinGBitBlt");
			(FARPROC)__WinGStretchBlt =
						pWinGModule->GetProcAddress("WinGStretchBlt");

			if (!__WinGCreateDC ||
				!__WinGRecommendDIBFormat ||
				!__WinGCreateBitmap ||
				!__WinGGetDIBPointer ||
				!__WinGGetDIBColorTable ||
				!__WinGSetDIBColorTable ||
				!__WinGCreateHalftonePalette ||
				!__WinGCreateHalftoneBrush ||
				!__WinGBitBlt ||
				!__WinGStretchBlt)
			{
				delete pWinGModule;

				THROW(TModule::TXInvalidModule());
			}
		}
		catch (TModule::TXInvalidModule)
		{
			pWinGModule = 0;

			::MessageBox(0,"Bad or Missing WING.DLL","Error",
														MB_OK | MB_ICONHAND);

			THROW(TModule::TXInvalidModule());
		}
	}
	++nWinGModuleRefCount;

	Handle = __WinGCreateDC(); // Change the TDC handle to the WinGDC
	CheckValid();
}


//-----------------------
//  TWinGDC::~TWinGDC()
//-----------------------
TWinGDC::~TWinGDC()
{
	RestoreObjects();
	if (ShouldDelete)  ::DeleteDC((HDC)Handle);
	Handle = 0;

	--nWinGModuleRefCount;
	if (!nWinGModuleRefCount)
	{
		delete pWinGModule;
		pWinGModule = 0;
	}
}


